/*
	Away System Modification - A new Away System for Miranda IM
	Copyright (C) 2004-2005 Iksaif Entertainment
	Copyright (C) 2002-2003 Goblineye Entertainment

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AwaySys.h"

static int g_iDefMsg = 0; // default message
static bool g_fReset = false;
static HWND g_hwndOptsDlg = NULL;
HWND g_hwndMoreOpts = NULL;



INT_PTR CALLBACK VarsListDlgProc(HWND hwndDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
	static HFONT s_hDrawFont;
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			s_hDrawFont = NULL;
			// ok now let's also setup the drawing font
			HFONT hFont = (HFONT)SendMessage(GetDlgItem(hwndDlg,IDC_VARSLIST_STATIC_TIME),WM_GETFONT,0,0); // try getting the font used by the control

			if (hFont == NULL) // if none...
			{
				hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT); // get the default font used by the GUI
			}

			LOGFONT logFont; // font information
			GetObject(hFont,sizeof(logFont),&logFont); // get font information
			logFont.lfUnderline = true; // underline it
			s_hDrawFont = CreateFontIndirect(&logFont); // recreate the font, underlined


			HWND hWorkWnd = GetWindow(hwndDlg,GW_CHILD); // first child
			while (hWorkWnd != NULL)
			{
				char szTemp[2]; // only get first char
				GetWindowText(hWorkWnd,szTemp,2);
				if (szTemp[0] == '%') // a var (this is a bit of a workaround, but it works)
				{
					SendMessage(hWorkWnd,WM_SETFONT,(WPARAM)s_hDrawFont,true);
					SetWindowLongPtr(hWorkWnd,GWLP_USERDATA,1); // mark
				}
				else // I don't trust the SDK or MS
				{
					SetWindowLongPtr(hWorkWnd,GWLP_USERDATA,0);
				}

				hWorkWnd = GetNextWindow(hWorkWnd,GW_HWNDNEXT);
			}

			// why is this down here? this way we can be sure that the descriptive text never starts with a '%'
			// this is a long shot, but I hate stupid bugs
			TranslateDialogDefault(hwndDlg);
			return(true);
		} break;
		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch(LOWORD(wParam))
					{
						case IDCANCEL:
						{
							DestroyWindow(hwndDlg);
							return(true);
						} break;
					}
				} break;
			}
		} break;
		case WM_CTLCOLORSTATIC:
		{
			if (GetWindowLongPtr((HWND)lParam,GWLP_USERDATA))
			{
				HDC hDC = (HDC)wParam;
				SetBkMode(hDC,TRANSPARENT);
				SetTextColor(hDC,RGB(50,50,50));
				return((INT_PTR)GetStockObject(NULL_BRUSH));
			}
		} break;
		case WM_CLOSE:
		{
			DestroyWindow(hwndDlg);
			return(true);
		} break;
		case WM_DESTROY:
		{
			if (s_hDrawFont != NULL)
			{
				DeleteObject(s_hDrawFont);
			}
			return(false);
		} break;
	}
	return(false);
}



int EnumMsgs(const char *szSetting,LPARAM lParam)
{
	ENUMMSGS_DATA *enumData = (ENUMMSGS_DATA*)lParam;
	
	if (strlen(szSetting) > 5) // at least 5
	{
		if (!strncmp(szSetting,"title",5)) // got
		{
			char szSuffix[DB_SUFFIX_MAX];

			for (unsigned int iLen=0;iLen < (strlen(szSetting)-5);iLen++)
				szSuffix[iLen] = *(szSetting+5+iLen);
			szSuffix[iLen] = '\0';

			// let's process the data from this message
			DBVARIANT dbVar;
			DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar); // start with the title
			if (dbVar.type == DBVT_ASCIIZ) // yippie (otherwise, this setting is invalid -> sux being a plugin <-)
			{
				int iPos = SendMessage(enumData->hComboBox,CB_ADDSTRING,0,(LPARAM)dbVar.pszVal);
				// goody, now let's deallocate
				DBFreeVariant(&dbVar); // free data

				// now for the data
				char szSet[64];
				strcpy(szSet,"msg");
				strcat(szSet,szSuffix);

				int iItemId = atoi(szSuffix);
				if ((iItemId-1) == enumData->iDefMsg) // this is the default msg
				{
					enumData->iDefMsg = iPos;
				}

				DBGetContactSetting(NULL,MOD_NAME,szSet,&dbVar); // the data

				if (dbVar.type != DBVT_ASCIIZ) // goofy!
				{
					SendMessage(enumData->hComboBox,CB_DELETESTRING,iPos,0);
					DBFreeVariant(&dbVar); // free data
					return(0); // sux being us :(
				}

				// else... we got a string
				char *szData;
				szData = new char[strlen(dbVar.pszVal)+1];
				strcpy(szData,dbVar.pszVal);

				DBFreeVariant(&dbVar);

				SendMessage(enumData->hComboBox,CB_SETITEMDATA,iPos,(LPARAM)szData); // associate the data
			}
		}

	}

	return(1);
}

int EnumLastMsgs(const char *szSetting,LPARAM lParam)
{
	ENUMMSGS_DATA *enumData = (ENUMMSGS_DATA*)lParam;

	if (strlen(szSetting) > 4) // at least 4
	{
		if (!strncmp(szSetting,"rmsg",4)) // got
		{
			char szSuffix[DB_SUFFIX_MAX];
			
			for (unsigned int iLen=0;iLen < (strlen(szSetting)-4);iLen++)
				szSuffix[iLen] = *(szSetting+4+iLen);
			szSuffix[iLen] = '\0';
		
			// let's process the data from this message
			DBVARIANT dbVar;
			DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar); // get the msg
			if(dbVar.type == DBVT_ASCIIZ) // yippie (otherwise, this setting is invalid -> sux being a plugin <-)
			{
				char *szData;
				szData = new char[strlen(dbVar.pszVal)+1];
				strcpy(szData,dbVar.pszVal);
				
				char szTitle[AWAY_TITLE_MAX+1];
				
				if(strlen(szData) > AWAY_TITLE_MAX)
				{
					strncpy(szTitle,szData,(AWAY_TITLE_MAX - 3));
					szTitle[AWAY_TITLE_MAX-3]='\0';
					strcat(szTitle,"...");
				}
				else
				{
					strcpy(szTitle,szData);
				}

				int iPos = SendMessage(enumData->hComboBox,CB_ADDSTRING,0,(LPARAM)szTitle);

				DBFreeVariant(&dbVar);
				
				SendMessage(enumData->hComboBox,CB_SETITEMDATA,iPos,(LPARAM)szData); // associate the data
			
			}
		}

	}

	return(1);
}

static int Apply_Changes(HWND hOptDlg,HWND hMoreOptDlg)
{
	// save settings
	// defmsg shouldn't be saved if we don't have any msgs present

	byte bLMEStat;
	byte bSDLGPos;
	if (hMoreOptDlg != NULL)
	{
		bLMEStat = DBGetContactSettingByte(NULL,MOD_NAME,"lmestat",0);
		bSDLGPos = DBGetContactSettingByte(NULL,MOD_NAME,"sdlgpos",0);
	}

	Delete_Settings(hOptDlg != NULL,hMoreOptDlg != NULL,false,false); // delete previous settings that might have changed

	if (hOptDlg != NULL)
	{
	   int iLen = GetWindowTextLength(GetDlgItem(hOptDlg, IDC_OPTDLG_PREFIX));
       if (iLen)
       {
           char* sPrefix = new char[iLen+1];
           GetDlgItemText(hOptDlg, IDC_OPTDLG_PREFIX, sPrefix, iLen+1);
           DBWriteContactSettingString(NULL, MOD_NAME, "prefix", sPrefix);
           delete [] sPrefix;
       } 
	   
	   DBWriteContactSettingByte(NULL,MOD_NAME,"usedef",(IsDlgButtonChecked(hOptDlg,IDC_OPTDLG_USEDEFAULT) == BST_CHECKED) ? (1) : (0));
	   DBWriteContactSettingByte(NULL,MOD_NAME,"defnick",(IsDlgButtonChecked(hOptDlg,IDC_OPTDLG_USEDEFNICK) == BST_CHECKED) ? (1) : (0));

		int iCount = SendDlgItemMessage(hOptDlg,IDC_OPTDLG_MSGNAME,CB_GETCOUNT,0,0);

		for (int index=0;index < iCount;index++)
		{
			char szSuffix[DB_SUFFIX_MAX];
			char szSetting[128];

			sprintf(szSuffix,"%d",index+1);
			strcpy(szSetting,"title");
			strcat(szSetting,szSuffix);

			char szValue[AWAY_PRED_NAME_MAX+1];
			SendDlgItemMessage(hOptDlg,IDC_OPTDLG_MSGNAME,CB_GETLBTEXT,index,(LPARAM)szValue);
			DBWriteContactSettingString(NULL,MOD_NAME,szSetting,szValue);

			strcpy(szSetting,"msg");
			strcat(szSetting,szSuffix);

			char *szMsg = (char*)SendDlgItemMessage(hOptDlg,IDC_OPTDLG_MSGNAME,CB_GETITEMDATA,index,0);
			DBWriteContactSettingString(NULL,MOD_NAME,szSetting,szMsg);
		}

		if (iCount > 0)
			DBWriteContactSettingWord(NULL,MOD_NAME,"defmsg",g_iDefMsg);
	}

	if (hMoreOptDlg != NULL)
	{
		// delay
		if (IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_NEVERSTATDLG) == BST_UNCHECKED) // then we need to get it and update and stuff
		{
			char szResult[5];
			int iResult;

			GetDlgItemText(hMoreOptDlg,IDC_MOREOPTDLG_WAITFORMSG,szResult,5);
			if (strlen(szResult) > 0) // we got a value, otherwise -> don't set this
			{
				iResult = atoi(szResult);
				DBWriteContactSettingWord(NULL,MOD_NAME,"clsdelay",iResult);
			}
		}

		byte bWantLMEStat = (IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_LMESTAT) == BST_CHECKED) ? (1) : (0);

		if (bLMEStat != bWantLMEStat) // if we're switching to a new state
		{
			if ((bLMEStat != 1) && (bLMEStat != 0))
			{
				bLMEStat = 0;
			}

			if (bLMEStat == 1) // already got individ settings, kill 'em all
			{
				// go through all contacts and delete all
				HANDLE hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
				while (hContact != NULL)
				{
					DBDeleteContactSetting(hContact,MOD_NAME,"Awaylmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"Nalmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"Dndlmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"Occupiedlmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"FreeChatlmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"Otplmsg");
					DBDeleteContactSetting(hContact,MOD_NAME,"Otllmsg");
					hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact,0);
				}
				DBDeleteContactSetting(NULL,MOD_NAME,"Awaylmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"Nalmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"Dndlmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"Occupiedlmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"FreeChatlmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"Otplmsg");
				DBDeleteContactSetting(NULL,MOD_NAME,"Otllmsg");
			}
			else if (bLMEStat == 0)  // already got individ settings
			{
				// go through all contacts and delete all
				HANDLE hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
				while (hContact != NULL)
				{
					DBDeleteContactSetting(hContact,MOD_NAME,"lmsg");
					hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact,0);
				}
				DBDeleteContactSetting(NULL,MOD_NAME,"lmsg");
			}
		}
		DBWriteContactSettingByte(NULL,MOD_NAME,"lmestat",bWantLMEStat);


		// dialog position
		byte bWantDLGSPos = (IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_RMDLGPOS) == BST_CHECKED) ? (1) : (0);
		
		if ((bSDLGPos != bWantDLGSPos) && (bSDLGPos == 1)) // if we're changing back, be sure to delete the current config
		{
			DBDeleteContactSetting(NULL,MOD_NAME,"dlgposx");
			DBDeleteContactSetting(NULL,MOD_NAME,"dlgposy");
		}

		DBWriteContactSettingByte(NULL,MOD_NAME,"sdlgpos",bWantDLGSPos);

		// type
		DBWriteContactSettingByte(NULL,MOD_NAME,"sendtype",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SENDONCE) == BST_CHECKED) ? (1) : (0));
		DBWriteContactSettingByte(NULL,MOD_NAME,"sendmsg",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SENDMSG) == BST_CHECKED) ? (1) : (0));
		UpdateSOEButtons((IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SENDMSG) == BST_CHECKED) ? (1) : (0));
		DBWriteContactSettingByte(NULL,MOD_NAME,"sendnoicq",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SENDNONICQ) == BST_CHECKED) ? (1) : (0));
		DBWriteContactSettingByte(NULL,MOD_NAME,"logsendedmsg",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_LOGMSG) == BST_CHECKED) ? (1) : (0));
		
		// event mask
		DBWriteContactSettingByte(NULL,MOD_NAME,"sendevent",((IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_EVNTMSG) == BST_CHECKED) ? (SENDSMSG_EVENT_MSG) : (0)) | ((IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_EVNTURL) == BST_CHECKED) ? (SENDSMSG_EVENT_URL) : (0)) | ((IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_EVNTFILE) == BST_CHECKED) ? (SENDSMSG_EVENT_FILE) : (0)));

		// set last
		DBWriteContactSettingByte(NULL,MOD_NAME,"slastcnt",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SETLASTCONTCAT) == BST_CHECKED) ? (1) : (0));

		// menu item
		DBWriteContactSettingByte(NULL,MOD_NAME,"enablemi",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_USEMENUITEM) == BST_CHECKED) ? (1) : (0));

		// recent msg
		DBWriteContactSettingByte(NULL,MOD_NAME,"showrmsg",(IsDlgButtonChecked(hMoreOptDlg,IDC_MOREOPTDLG_SHOWRMSG) == BST_CHECKED) ? (1) : (0));

		// status' we ignore
		DWORD dwTemp = 0;
		for (int index=0, iCount=SendDlgItemMessage(hMoreOptDlg,IDC_MOREOPTDLG_NOPOPLIST,CB_GETCOUNT,0,0);index < iCount;index++)
		{
			DWORD dwUserData = SendDlgItemMessage(hMoreOptDlg,IDC_MOREOPTDLG_NOPOPLIST,CB_GETITEMDATA,index,0);
			if (dwUserData & 0x10000)
			{
				// find index in list of status' (we might have not added one or two from the list... probably more)
				for (int iStatusPos = 0;iStatusPos < sizeof(statusModeList) / sizeof(statusModeList[0]);iStatusPos++)
				{
					if ((dwUserData & 0xFFFF) == statusModeList[iStatusPos]) // got it
					{
						break;
					}
				}

				// no error checking here, it's not possible... I hope :)
				// important! we save by position of the statusModeList array (const), not by position in combobox (too dynamic)!!
				dwTemp |= 1 << iStatusPos;
			}
		}
		if (dwTemp) DBWriteContactSettingDword(NULL,MOD_NAME,"nodlgs",dwTemp);

		dwTemp = 0;
		for (index=0, iCount=SendDlgItemMessage(hMoreOptDlg,IDC_MOREOPTDLG_NOSSMLIST,CB_GETCOUNT,0,0);index < iCount;index++)
		{
			// for second list
			DWORD dwUserData = SendDlgItemMessage(hMoreOptDlg,IDC_MOREOPTDLG_NOSSMLIST,CB_GETITEMDATA,index,0);
			if (dwUserData & 0x10000)
			{
				// find index in list of status' (we might have not added one or two from the list... probably more)
				for (int iStatusPos = 0;iStatusPos < sizeof(statusModeList) / sizeof(statusModeList[0]);iStatusPos++)
				{
					if ((dwUserData & 0xFFFF) == statusModeList[iStatusPos]) // got it
					{
						break;
					}
				}

				// no error checking here, it's not possible... I hope :)
				// important! we save by position of the statusModeList array (const), not by position in combobox (too dynamic)!!
				dwTemp |= 1 << iStatusPos;
			}
		}
		if (dwTemp) DBWriteContactSettingDword(NULL,MOD_NAME,"nossm",dwTemp);
	}

	return(1);
}



int CALLBACK OptDlgMain(HWND hwndDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch (msg)
	{
		case WM_INITDIALOG:
		{
			TranslateDialogDefault(hwndDlg);

			HWND hComboBox = GetDlgItem(hwndDlg,IDC_OPTDLG_MSGNAME);

			g_hwndOptsDlg = hwndDlg;
			
			Check_Settings(); // make sure we have valid settings

			EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),false);
			EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),false);

			SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGDATA,EM_LIMITTEXT,1024,0);
			SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_LIMITTEXT,64,0);
			SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,IDC_OPTDLG_PREFIX,64,0);
			
			CheckRadioButton(hwndDlg,IDC_OPTDLG_USELASTMSG,IDC_OPTDLG_USEDEFAULT,(DBGetContactSettingByte(NULL,MOD_NAME,"usedef",0) == 1) ? (IDC_OPTDLG_USEDEFAULT) : (IDC_OPTDLG_USELASTMSG));
			
			DBVARIANT dbvi;
			if (DBGetContactSetting(NULL, "ICQ","Nick",&dbvi))
			{
				//strcpy(myNick, dbv.pszVal );
				EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_USEDEFNICK),false);
				DBFreeVariant(&dbvi); // important, free the memory
			}
			else if (DBGetContactSettingByte(NULL,MOD_NAME,"defnick",1))
			{
				CheckDlgButton(hwndDlg,IDC_OPTDLG_USEDEFNICK,BST_CHECKED);
			}

			DBVARIANT dbv;
			if (DBGetContactSetting(NULL,MOD_NAME,"prefix",&dbv) == 0 && dbv.type==DBVT_ASCIIZ) {
				SetDlgItemText( hwndDlg, IDC_OPTDLG_PREFIX, dbv.pszVal );
				DBFreeVariant(&dbv); // important, free the memory
			}  
			
			g_iDefMsg = (int)DBGetContactSettingWord(NULL,MOD_NAME,"defmsg",0); // try finding a default msg in any case
			

			ENUMMSGS_DATA *enumData = new ENUMMSGS_DATA;
			if (enumData == NULL)
			{
				MessageBox(hwndDlg,Translate("An error has occured while trying to acquire messages"),Translate("Away System"),MB_ICONWARNING);
				return(false);
			}
		
			enumData->hComboBox = hComboBox;
			enumData->iDefMsg = g_iDefMsg;

			// let's get everything from the database, we'll pass the combobox's handle
			DBCONTACTENUMSETTINGS dbEnum;
			dbEnum.lParam = (LPARAM)enumData;
			dbEnum.ofsSettings = 0;
			dbEnum.pfnEnumProc = EnumMsgs;
			dbEnum.szModule = MOD_NAME;
			CallService(MS_DB_CONTACT_ENUMSETTINGS,NULL,(LPARAM)&dbEnum);

			g_iDefMsg = enumData->iDefMsg; // get it back

			delete enumData;

			if (SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCOUNT,0,0) == 0) // no msgs, sux
				CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_UNCHECKED); // do what's needed
			else
			{
				// do what's also needed
				// get the default, set it in the combo box+data+tick the thingy+other stuff

				if (g_iDefMsg > SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCOUNT,0,0)-1) // error (invalid def msg)
					g_iDefMsg = 0;
				SendMessage(hComboBox,CB_SETCURSEL,g_iDefMsg,0); // select default
				CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_CHECKED); // check default button

				SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGDATA,(char*)SendMessage(hComboBox,CB_GETITEMDATA,g_iDefMsg,0)); // set the data

				// delete button
				EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),true);
			}
			return(true);
		} break;
		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch (LOWORD(wParam))
					{
						case IDC_OPTDLG_USEDEFNICK:
						case IDC_OPTDLG_USELASTMSG:
						case IDC_OPTDLG_USEDEFAULT:
						{
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
							return(0);
						} break;
						case IDC_OPTDLG_VARSLIST: // vars list
						{
							CreateDialog(g_hInstance,MAKEINTRESOURCE(IDD_VARSLISTDLG),hwndDlg,VarsListDlgProc);
						} break;
						case IDC_OPTDLG_RESET: // reset all settings (probably wants to uninstall/smt went wrong) :(
						{
							if (MessageBox(hwndDlg,Translate("Are you sure you would like to delete settings? If you continue, all of your settings will be deleted!"),Translate("Away System"),MB_YESNO | MB_DEFBUTTON2 | MB_ICONWARNING) == IDYES)
							{
								g_fReset = true;
								// disable all windows
								Disable_ChildWindows(hwndDlg);
								Disable_ChildWindows(g_hwndMoreOpts);

								MessageBox(hwndDlg,Translate("All settings will be erased after you close the options window!"),Translate("Away System"),0);
							}
						} break;
						case IDC_OPTDLG_MAKEDEFAULT: // default button was pressed
						{
							// don't let the current one to be changed
							// if another is changed, make sure to change it in the global var

							// guidelines: if save button is enabled, change it
							if ((!IsWindowEnabled(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE))) && (g_iDefMsg != -1)) // -1 means we got no msgs
							{
								if (g_iDefMsg != SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCURSEL,0,0)) // if another
									CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_CHECKED); // tick

								g_iDefMsg = SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCURSEL,0,0);

								SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
							}
							return(0);
						} break;
						case IDC_OPTDLG_SAVE: // save button
						{
							// no checking is needed, just save the msg
							// check if we already have the title, if so - update
							// if we got 0 msgs and we're adding 1 new -> make it default

							HWND hwndCB = GetDlgItem(hwndDlg,IDC_OPTDLG_MSGNAME);
							int iCount = SendMessage(hwndCB,CB_GETCOUNT,0,0);
							
							char szTitle[AWAY_PRED_NAME_MAX+1];
							SendMessage(hwndCB,WM_GETTEXT,AWAY_PRED_NAME_MAX+1,(LPARAM)szTitle);
							if (strlen(szTitle) == 0)
								MessageBox(hwndDlg,Translate("Please enter a title for the message before saving."),Translate("Away System"),MB_ICONWARNING);
							else
							{
								char szMsg[AWAY_PRED_DATA_MAX+1];
								SendMessage(GetDlgItem(hwndDlg,IDC_OPTDLG_MSGDATA),WM_GETTEXT,AWAY_PRED_DATA_MAX+1,(LPARAM)szMsg);

								if (strlen(szMsg) == 0)
									MessageBox(hwndDlg,Translate("Please enter a message before saving."),Translate("Away System"),MB_ICONWARNING);
								else
								{
									int iPos;
									// k, save
									iPos = SendMessage(hwndCB,CB_FINDSTRINGEXACT,-1,(LPARAM)szTitle);
									if (iPos == CB_ERR) // this item is new
									{
										iPos = SendMessage(hwndCB,CB_ADDSTRING,0,(LPARAM)szTitle);

										if (iPos <= g_iDefMsg) // if we replaced the current def msg, or somewhere before
										{
											g_iDefMsg++; // push down (the list is sorted)
										}
									}
									else
									{
										// need to deallocate what was left behind
										char *lpszMessage = (char*)SendMessage(hwndCB,CB_GETITEMDATA,iPos,0);
										if (lpszMessage != NULL) // to be sure
											delete [] lpszMessage;
									}
									
									// else... we got a string
									char *lpszData;
									lpszData = new char[strlen(szMsg)+1];
									strcpy(lpszData,szMsg);

									SendMessage(hwndCB,CB_SETITEMDATA,iPos,(LPARAM)lpszData); // associate the data

									if (iCount == 0) // had nothing before
										g_iDefMsg = 0;

									SendMessage(hwndCB,CB_SETCURSEL,iPos,0); // select the last one added (our new baby!)

									if (iPos == g_iDefMsg)
										CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_CHECKED); // check default button

									SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGDATA,lpszData); // set the data

									// delete button
									EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),true);
									EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),false);

									SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
								}
							}
							return(0);
						} break;
						case IDC_OPTDLG_DEL: // delete button
						{
							// no checking... just del the msg =P
							int iSel = SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCURSEL,0,0);
							if (iSel != CB_ERR)
							{
								char *lpszMsg = (char*)SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETITEMDATA,iSel,0);
								if (lpszMsg != NULL)
									delete [] lpszMsg;
								SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_DELETESTRING,iSel,0);

								if ((iSel <= g_iDefMsg) && (g_iDefMsg > 0)) // pick the one before (we don't have a prob if it's 0)
									g_iDefMsg--;

								EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),false);
								if (SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCOUNT,0,0) == 0)
								{
									g_iDefMsg = -1;
									SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGNAME,"");
									SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGDATA,"");
									EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),false);
									CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_UNCHECKED); // check default button
								}
								else
								{
									SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_SETCURSEL,g_iDefMsg,0); // select default
									CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_CHECKED); // check default button
									SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGDATA,(char*)SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETITEMDATA,g_iDefMsg,0)); // set the data

									// delete button
									EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),true);
								}
								SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
							}
							return(0);
						} break;
						default:break;
					}
				} break;
				case CBN_SELCHANGE: // user changed the selection, let's use a saved msg
				{
					if (LOWORD(wParam) == IDC_OPTDLG_MSGNAME)
					{
						// here we enable delete and disable save, we also tick/un-tick default accordingly
						int iSelect = SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETCURSEL,0,0); // get new msg num
						if (iSelect != CB_ERR)
						{
							SetDlgItemText(hwndDlg,IDC_OPTDLG_MSGDATA,(char*)SendDlgItemMessage(hwndDlg,IDC_OPTDLG_MSGNAME,CB_GETITEMDATA,iSelect,0)); // set the data	
							CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,(iSelect == g_iDefMsg) ? (BST_CHECKED) : (BST_UNCHECKED)); // check default button
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),true);
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),false);
						}
					}
					return(0);
				} break;
				case CBN_EDITUPDATE: // user changed the text, meaning he's starting something new, amazing
				{
					switch (LOWORD(wParam))
					{
						case IDC_OPTDLG_MSGNAME:
						{
							// doesn't matter if what we have here is present:
							// only enable save, and disable delete
							// also un-tick default
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),true);
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),false);
							CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_UNCHECKED); // check default button
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						} break;
						default:break;
					}
					return(0);
				} break;
				case EN_UPDATE: // some text is being edited in the edit control
				{
					switch (LOWORD(wParam))
					{
						case IDC_OPTDLG_PREFIX:
						{
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						} break;
						case IDC_OPTDLG_MSGDATA:
						{
							// enable the save button (make sure), so saving can be done
							// also disable delete
							// also un-tick default
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_SAVE),true);
							EnableWindow(GetDlgItem(hwndDlg,IDC_OPTDLG_DEL),false);
							CheckDlgButton(hwndDlg,IDC_OPTDLG_MAKEDEFAULT,BST_UNCHECKED); // check default button
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						}
						default:break;
					}
				} break;
				default:break;
			}
		} break;
		case WM_NOTIFY:
		{
			NMHDR* nmhdr = (NMHDR*)lParam;
			switch (nmhdr->code)
			{
				case PSN_APPLY:
				{
					if (!g_fReset) // save settings
						Apply_Changes(hwndDlg,g_hwndMoreOpts);
					return(true);
				} break;
			}
		} break;
		case WM_DESTROY:
		{
			// deallocate all data
			HWND hComboBox = GetDlgItem(hwndDlg,IDC_OPTDLG_MSGNAME);
			int iCount = SendMessage(hComboBox,CB_GETCOUNT,0,0);
			
			for (int index=0;index < iCount;index++)
			{
				char *lpszData = (char*)SendMessage(hComboBox,CB_GETITEMDATA,index,0);
				if (lpszData != NULL) // no (), just to be sure
					delete [] lpszData; // deallocate!
			}

			if (g_fReset) // reset settings
				Delete_Settings(true,true,true,true); // all settings go

			g_hwndOptsDlg = NULL;
			g_fReset = false;
			return(0);
		} break;
	}
	return(false);
}



int CALLBACK MoreOptDlg(HWND hwndDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
	static bool s_fRedraw;
	static HICON s_hMsgIcon = NULL, s_hURLIcon = NULL, s_hFileIcon = NULL; // event icons.

	switch (msg)
	{
		case WM_INITDIALOG:
		{
			TranslateDialogDefault(hwndDlg);

			s_fRedraw = false;
			g_hwndMoreOpts = hwndDlg;

			HWND hDlgWait = GetDlgItem(hwndDlg,IDC_MOREOPTDLG_WAITFORMSG);
			SendMessage(hDlgWait,EM_LIMITTEXT,4,0);

			Check_Settings(); // make sure we have valid settings

			bool fDis;

			int iDelay = DBGetContactSettingWord(NULL,MOD_NAME,"clsdelay",0);
			if (iDelay == 0) // no delay
			{
				// tick box, disable sub-controls
				// a precaution, so we won't have any probs here
				char szTemp[1];
				szTemp[0] = '\0';
				SendMessage(hDlgWait,WM_SETTEXT,0,(LPARAM)szTemp);

				fDis = false;
			}
			else // delay!
			{
				// un-tick box, enable sub-controls, set #
				fDis = true;
				SetDlgItemInt(hwndDlg,IDC_MOREOPTDLG_WAITFORMSG,iDelay,true);
			}

			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_NEVERSTATDLG,(!fDis) ? (BST_CHECKED) : (BST_UNCHECKED));
			EnableWindow(hDlgWait,fDis);
			EnableWindow(GetDlgItem(hwndDlg,IDC_STATIC_MOREOPTDLG_WAIT),fDis);
			EnableWindow(GetDlgItem(hwndDlg,IDC_STATIC_MOREOPTDLG_SECONDSREMAIN),fDis);


			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_LMESTAT,(DBGetContactSettingByte(NULL,MOD_NAME,"lmestat",0) == 1) ? (BST_CHECKED) : (BST_UNCHECKED));
			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_RMDLGPOS,(DBGetContactSettingByte(NULL,MOD_NAME,"sdlgpos",0) == 1) ? (BST_CHECKED) : (BST_UNCHECKED));

			SendMessage(hwndDlg,WM_USER + 20,0,0);

			// send type
			CheckRadioButton(hwndDlg,IDC_MOREOPTDLG_SENDONCE,IDC_MOREOPTDLG_SENDALL,(DBGetContactSettingByte(NULL,MOD_NAME,"sendtype",1) == 1) ? (IDC_MOREOPTDLG_SENDONCE) : (IDC_MOREOPTDLG_SENDALL));

			// send non-icq
			if (DBGetContactSettingByte(NULL,MOD_NAME,"sendnoicq",1))
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_SENDNONICQ,BST_CHECKED);
			}
			
			// send non-icq
			if (DBGetContactSettingByte(NULL,MOD_NAME,"sendmsg",1))
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_SENDMSG,BST_CHECKED);
			}

			// log message i send
			if (DBGetContactSettingByte(NULL,MOD_NAME,"logsendedmsg",1))
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_LOGMSG,BST_CHECKED);
			}

			// events
			WORD wEventMask = DBGetContactSettingByte(NULL,MOD_NAME,"sendevent",0);

			if (wEventMask & SENDSMSG_EVENT_MSG)
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_EVNTMSG,BST_CHECKED);
			}
			if (wEventMask & SENDSMSG_EVENT_URL)
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_EVNTURL,BST_CHECKED);
			}
			if (wEventMask & SENDSMSG_EVENT_FILE)
			{
				CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_EVNTFILE,BST_CHECKED);
			}

			// slatcnt
			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_SETLASTCONTCAT,(DBGetContactSettingByte(NULL,MOD_NAME,"slastcnt",0)) ? (BST_CHECKED) : (BST_UNCHECKED));
			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_USEMENUITEM,(DBGetContactSettingByte(NULL,MOD_NAME,"enablemi",1)) ? (BST_CHECKED) : (BST_UNCHECKED));
			CheckDlgButton(hwndDlg,IDC_MOREOPTDLG_SHOWRMSG,(DBGetContactSettingByte(NULL,MOD_NAME,"showrmsg",1)) ? (BST_CHECKED) : (BST_UNCHECKED));

			if (g_fReset) // disable all
			{
				Disable_ChildWindows(hwndDlg);
			}

			// types of msgs we don't touch
			DWORD dwProtoModeMsgFlags = 0;
			PROTOCOLDESCRIPTOR **proto;

			int iProtoCount = 0;
			CallService(MS_PROTO_ENUMPROTOCOLS,(WPARAM)&iProtoCount,(LPARAM)&proto);
			for(int index = 0;index < iProtoCount;index++)
			{
				dwProtoModeMsgFlags |= CallProtoService(proto[index]->szName,PS_GETCAPS,PFLAGNUM_3,0); // get types
			}

			for (index = 0;index < sizeof(statusModeList) / sizeof(statusModeList[0]);index++)
			{
				// data: lowest 16 bit, the status id
				// 0x10000: 1 - don't show dlg, 0 - show
				// this is done twice for two comboboxes (no pop list, and no ssm)
				// SSM
				// SSM is used for all status'
				char *pszDesc = (char*)CallService(MS_CLIST_GETSTATUSMODEDESCRIPTION,statusModeList[index],0);
				if (statusModeList[index] != ID_STATUS_OFFLINE) // no offline
				{
					int iPos = SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOSSMLIST,CB_ADDSTRING,0,(LPARAM)pszDesc);
					SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOSSMLIST,CB_SETITEMDATA,iPos,statusModeList[index] | (((DBGetContactSettingDword(NULL,MOD_NAME,"nossm",SSM_DEFAULT) >> index) & 1) << 16));
				}

				// POP
				if (!(dwProtoModeMsgFlags & Proto_Status2Flag(statusModeList[index]))) continue; // nope.
				int iPos = SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOPOPLIST,CB_ADDSTRING,0,(LPARAM)pszDesc);
				SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOPOPLIST,CB_SETITEMDATA,iPos,statusModeList[index] | (((DBGetContactSettingDword(NULL,MOD_NAME,"nodlgs",0) >> index) & 1) << 16));
			}
			SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOPOPLIST,CB_SETCURSEL,0,0); // select first (we always have more than one, we made sure on startup)
			SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_NOSSMLIST,CB_SETCURSEL,0,0); // select first
			SendMessage(hwndDlg,WM_USER+22,0,0); // for no pop list
			SendMessage(hwndDlg,WM_USER+23,0,0); // for no ssm list

			s_fRedraw = true;
			return(true);
		} break;
		case WM_USER + 20:
		{
			// send stat msg related stuff
			// this way (CopyImage()) was suggested by Corsario, tnx man :)), to fix the 32x32 icons issue
			SendMessage(hwndDlg,WM_USER+21,0,0); // delete icons
			s_hMsgIcon = (HICON)CopyImage(LoadSkinnedIcon(SKINICON_EVENT_MESSAGE),IMAGE_ICON,16,16,LR_COPYFROMRESOURCE);
			s_hURLIcon = (HICON)CopyImage(LoadSkinnedIcon(SKINICON_EVENT_URL),IMAGE_ICON,16,16,LR_COPYFROMRESOURCE);
			s_hFileIcon = (HICON)CopyImage(LoadSkinnedIcon(SKINICON_EVENT_FILE),IMAGE_ICON,16,16,LR_COPYFROMRESOURCE);
			
			SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_EVNTMSG,BM_SETIMAGE,IMAGE_ICON,(LPARAM)s_hMsgIcon);
			SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_EVNTURL,BM_SETIMAGE,IMAGE_ICON,(LPARAM)s_hURLIcon);
			SendDlgItemMessage(hwndDlg,IDC_MOREOPTDLG_EVNTFILE,BM_SETIMAGE,IMAGE_ICON,(LPARAM)s_hFileIcon);
			return(true);
		} break;
		case WM_USER + 21:
		{
			if (s_hMsgIcon != NULL)
			{
				DestroyIcon(s_hMsgIcon);
				s_hMsgIcon = NULL;
			}
			if (s_hURLIcon != NULL)
			{
				DestroyIcon(s_hURLIcon);
				s_hURLIcon = NULL;
			}
			if (s_hFileIcon != NULL)
			{
				DestroyIcon(s_hFileIcon);
				s_hFileIcon = NULL;
			}
			return(true);
		} break;
		case WM_USER + 22: // change in combo box of status' (pop list)
		case WM_USER + 23: // change in combo box of status' (ssm list)
		{
			WORD wID = (msg == (WM_USER+22)) ? (IDC_MOREOPTDLG_NOPOPLIST) : (IDC_MOREOPTDLG_NOSSMLIST);
			// not my style of coding, but doing this in one line is way too confusing :)
			int iPos = SendDlgItemMessage(hwndDlg,wID,CB_GETCURSEL,0,0);
			DWORD dwData = SendDlgItemMessage(hwndDlg,wID,CB_GETITEMDATA,iPos,0);
			CheckDlgButton(hwndDlg,(wID == IDC_MOREOPTDLG_NOPOPLIST) ? (IDC_MOREOPTDLG_NOPOP) : (IDC_MOREOPTDLG_NOSSM),(dwData >> 16) ? (BST_CHECKED) : (BST_UNCHECKED));
			return(true);
		} break;
		case WM_NOTIFY:
		{
			NMHDR* nmhdr = (NMHDR*)lParam;
			switch (nmhdr->code)
			{
				case PSN_APPLY:
				{
					if (!g_fReset) // save settings
						Apply_Changes(g_hwndOptsDlg,hwndDlg);
					return(true);
				} break;
			}
		} break;
		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch (LOWORD(wParam))
					{
						case IDC_MOREOPTDLG_NOSSM: // change status for ssm
						case IDC_MOREOPTDLG_NOPOP: // changed status
						{
							WORD wID = (LOWORD(wParam) == IDC_MOREOPTDLG_NOPOP) ? (IDC_MOREOPTDLG_NOPOPLIST) : (IDC_MOREOPTDLG_NOSSMLIST);
							int iPos = SendDlgItemMessage(hwndDlg,wID,CB_GETCURSEL,0,0);
							DWORD dwUserData = SendDlgItemMessage(hwndDlg,wID,CB_GETITEMDATA,iPos,0);
							dwUserData = (dwUserData & 0xFFFF) | (~dwUserData & 0x10000);
							SendDlgItemMessage(hwndDlg,wID,CB_SETITEMDATA,iPos,dwUserData);
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
							// no need to update display (auto button)
						} break;
						case IDC_MOREOPTDLG_SENDONCE:
						case IDC_MOREOPTDLG_SENDALL:
						{
							CheckRadioButton(hwndDlg,IDC_MOREOPTDLG_SENDONCE,IDC_MOREOPTDLG_SENDALL,LOWORD(wParam));
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						} break;
						case IDC_MOREOPTDLG_USEMENUITEM:
						case IDC_MOREOPTDLG_SHOWRMSG:
						case IDC_MOREOPTDLG_SETLASTCONTCAT:
						case IDC_MOREOPTDLG_SENDNONICQ:
						case IDC_MOREOPTDLG_LOGMSG:
						case IDC_MOREOPTDLG_EVNTMSG:
						case IDC_MOREOPTDLG_EVNTURL:
						case IDC_MOREOPTDLG_EVNTFILE:
						case IDC_MOREOPTDLG_LMESTAT:
						case IDC_MOREOPTDLG_SENDMSG:
						case IDC_MOREOPTDLG_RMDLGPOS:
						{
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
							return(0);
						} break;
						case IDC_MOREOPTDLG_NEVERSTATDLG:
						{
							bool fSet;
							if (IsDlgButtonChecked(hwndDlg,IDC_MOREOPTDLG_NEVERSTATDLG) == BST_UNCHECKED)
							{
								fSet = true;
								char szTemp[5];
								GetDlgItemText(hwndDlg,IDC_MOREOPTDLG_WAITFORMSG,szTemp,5);
								if (strlen(szTemp) == 0) // no value
									SetDlgItemInt(hwndDlg,IDC_MOREOPTDLG_WAITFORMSG,(int)5,true);
							}
							else
								fSet = false;

							EnableWindow(GetDlgItem(hwndDlg,IDC_MOREOPTDLG_WAITFORMSG),fSet);
							EnableWindow(GetDlgItem(hwndDlg,IDC_STATIC_MOREOPTDLG_WAIT),fSet);
							EnableWindow(GetDlgItem(hwndDlg,IDC_STATIC_MOREOPTDLG_SECONDSREMAIN),fSet);

							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						} break;
						default:break;
					}
				} break;
				case CBN_SELCHANGE:
				{
					if (LOWORD(wParam) == IDC_MOREOPTDLG_NOPOPLIST) // cautious
					{
						SendMessage(hwndDlg,WM_USER+22,0,0); // update display
					}
					else if (LOWORD(wParam) == IDC_MOREOPTDLG_NOSSMLIST)
					{
						SendMessage(hwndDlg,WM_USER+23,0,0); // update display
					}
				} break;
				case EN_UPDATE:
				{
					if (LOWORD(wParam) == IDC_MOREOPTDLG_WAITFORMSG)
					{
						if (s_fRedraw)
						{
							SendMessage(GetParent(hwndDlg),PSM_CHANGED,(WPARAM)hwndDlg,0);
						}
					}
				} break;
				default:break;
			}
		} break;
		case WM_DESTROY:
		{
			SendMessage(hwndDlg,WM_USER+21,0,0); // delete icons
			g_hwndMoreOpts = NULL;
			g_fReset = false;
			return(0);
		} break;
	}
	return(false);
}



int OptsDlgInit(WPARAM wParam,LPARAM lParam)
{
	OPTIONSDIALOGPAGE optDi;
	memset(&optDi,0,sizeof(optDi));
	optDi.cbSize = sizeof(optDi);
	optDi.position = 920000000;
	optDi.pszTitle = Translate("Away System");
	optDi.pfnDlgProc = OptDlgMain;
	optDi.pszTemplate = MAKEINTRESOURCE(IDD_OPTDIALOGMAIN);
	optDi.hInstance = g_hInstance;
	optDi.flags = ODPF_BOLDGROUPS;

	CallService(MS_OPT_ADDPAGE,wParam,(LPARAM)&optDi);

	memset(&optDi,0,sizeof(optDi));
	optDi.cbSize = sizeof(optDi);
	optDi.position = 920000000;
	optDi.pszTitle = Translate("More Options");
	optDi.pfnDlgProc = MoreOptDlg;
	optDi.pszTemplate = MAKEINTRESOURCE(IDD_MOREOPTDIALOG);
	optDi.hInstance = g_hInstance;
	optDi.pszGroup = Translate("Away System");
	optDi.flags = ODPF_BOLDGROUPS;

	CallService(MS_OPT_ADDPAGE,wParam,(LPARAM)&optDi);
	return(0);
}



int Delete_Settings(bool bDelOpts,bool bDelMore,bool bDelLastMsg,bool fContactTempSets)
{
	// we need to delete settings
	if (bDelMore)
	{
		DBDeleteContactSetting(NULL,MOD_NAME,"clsdelay");
		DBDeleteContactSetting(NULL,MOD_NAME,"lmestat");
		DBDeleteContactSetting(NULL,MOD_NAME,"sdlgpos");
		// send msg
		DBDeleteContactSetting(NULL,MOD_NAME,"sendnoicq");
		DBDeleteContactSetting(NULL,MOD_NAME,"logsendedmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"sendtype");
		DBDeleteContactSetting(NULL,MOD_NAME,"sendevent");
		// new stuff
		DBDeleteContactSetting(NULL,MOD_NAME,"slastcnt");
		DBDeleteContactSetting(NULL,MOD_NAME,"enablemi");
		DBDeleteContactSetting(NULL,MOD_NAME,"showrmsg");

		// more new stuff
		DBDeleteContactSetting(NULL,MOD_NAME,"nodlgs");
		DBDeleteContactSetting(NULL,MOD_NAME,"nossm"); // send-status-message
	}
	if (bDelLastMsg)
	{
		DBDeleteContactSetting(NULL,MOD_NAME,"sendmsg");
		UpdateSOEButtons(0);
//		DBDeleteContactSetting(NULL,MOD_NAME,"ignorereq");
		//DBDeleteContactSetting(NULL,MOD_NAME,"cmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"ctime");
		DBDeleteContactSetting(NULL,MOD_NAME,"lmsg");
		// individ
		DBDeleteContactSetting(NULL,MOD_NAME,"Awaylmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"Nalmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"Dndlmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"Occupiedlmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"FreeChatlmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"Otplmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"Otllmsg");

		// extra stuff - dialog position
		DBDeleteContactSetting(NULL,MOD_NAME,"dlgposx");
		DBDeleteContactSetting(NULL,MOD_NAME,"dlgposy");

		// const stuff for contacts
		HANDLE hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
		while (hContact != NULL)
		{
			DBDeleteContactSetting(hContact,MOD_NAME,"sendmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"msgnum");
			DBDeleteContactSetting(hContact,MOD_NAME,"ignorereq");
			DBDeleteContactSetting(hContact,MOD_NAME,"lmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Awaylmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Nalmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Dndlmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Occupiedlmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"FreeChatlmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Otplmsg");
			DBDeleteContactSetting(hContact,MOD_NAME,"Otllmsg");
			hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact,0);
		}
	}
	if (bDelOpts)
	{
		DBDeleteContactSetting(NULL,MOD_NAME,"defmsg");
		DBDeleteContactSetting(NULL,MOD_NAME,"prefix");
		DBDeleteContactSetting(NULL,MOD_NAME,"usedef");
		DBDeleteContactSetting(NULL,MOD_NAME,"defnick");
		DBVARIANT dbVar;
		for (int index=1;;index++)
		{
			char szSuffix[DB_SUFFIX_MAX];
			char szSetting[128];

			sprintf(szSuffix,"%d",index);
			strcpy(szSetting,"title");
			strcat(szSetting,szSuffix);

			if (DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar) == 0) // exists
			{
				DBFreeVariant(&dbVar);
				DBDeleteContactSetting(NULL,MOD_NAME,szSetting);
				strcpy(szSetting,"msg");
				strcat(szSetting,szSuffix);
				DBDeleteContactSetting(NULL,MOD_NAME,szSetting);
			}
			else
				break;
		}
		for (int index=1;;index++)
		{
			char szSuffix[DB_SUFFIX_MAX];
			char szSetting[128];

			sprintf(szSuffix,"%d",index);
			strcpy(szSetting,"rmsg");
			strcat(szSetting,szSuffix);

			if (DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar) == 0) // exists
			{
				DBFreeVariant(&dbVar);
				DBDeleteContactSetting(NULL,MOD_NAME,szSetting);
			}
			else
				break;
		}
	}
	if (fContactTempSets) // all *temp* contact settings (mostly counts)
	{
		HANDLE hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
		while (hContact != NULL)
		{
			DBDeleteContactSetting(hContact,MOD_NAME,"sendcount");
			DBDeleteContactSetting(hContact,MOD_NAME,"msgnum");
			DBDeleteContactSetting(hContact,MOD_NAME,"cmsg");
			hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact,0);
		}
	}
	return(1);
}



static int Set_Init_Settings(void)
{
	Delete_Settings(true,true,true,true);

	// write initial settings
	DBWriteContactSettingString(NULL,MOD_NAME,"title1",Translate("Creepy"));
	DBWriteContactSettingString(NULL,MOD_NAME,"msg1",Translate("Your master, %mynick% is %statdesc% since the day that is only known as %day%... When he gets back, I'll tell him you dropped by..."));

	DBWriteContactSettingString(NULL,MOD_NAME,"title2",Translate("Gone fragging"));
	DBWriteContactSettingString(NULL,MOD_NAME,"msg2",Translate("Been fragging since %time%, I'll msg you later when the adrenaline wears off."));

	DBWriteContactSettingString(NULL,MOD_NAME,"title3",Translate(AWAY_DF_TITLE));
	DBWriteContactSettingString(NULL,MOD_NAME,"msg3",Translate(AWAY_DF_MSG));
	
	DBWriteContactSettingString(NULL,MOD_NAME,"title4",Translate("On the phone"));
	DBWriteContactSettingString(NULL,MOD_NAME,"msg4",Translate("I'm on the phone since %time%, give me a sec!"));

	DBWriteContactSettingWord(NULL,MOD_NAME,"defmsg",0); // default msg is now by default
	DBWriteContactSettingByte(NULL,MOD_NAME,"usedef",0);
	DBWriteContactSettingByte(NULL,MOD_NAME,"defnick",0);
	DBWriteContactSettingWord(NULL,MOD_NAME,"clsdelay",5); // default closing delay, if N/A, don't w8
	DBWriteContactSettingByte(NULL,MOD_NAME,"lmestat",0); // by default: not saving last msg for each status
	DBWriteContactSettingByte(NULL,MOD_NAME,"sdlgpos",0); // by default: not saving status message dialog position
	return(1);
}



int Check_Settings(void)
{
	if (DBGetContactSettingByte(NULL,MOD_NAME,"usedef",-1) == -1) // if either one doesn't exist
		Set_Init_Settings();
	else
	{
		if (DBGetContactSettingByte(NULL,MOD_NAME,"lmestat",233) == 233) // write lmestat if needed
			DBWriteContactSettingByte(NULL,MOD_NAME,"lmestat",0);

		if (DBGetContactSettingByte(NULL,MOD_NAME,"sdlgpos",233) == 233) // write sdlgpos if needed
			DBWriteContactSettingByte(NULL,MOD_NAME,"sdlgpos",0);
	}
	return(1);
}