/*
	Away System Modification - A new Away System for Miranda IM
	Copyright (C) 2004-2005 Iksaif Entertainment
	Copyright (C) 2002-2003 Goblineye Entertainment

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AwaySys.h"




char *StatusModeToDbSetting(int status,const char *suffix)
{
    char *prefix;
	static char str[64];

	switch(status) {
		case ID_STATUS_AWAY: prefix="Away";	break;
		case ID_STATUS_NA: prefix="Na";	break;
		case ID_STATUS_DND: prefix="Dnd"; break;
		case ID_STATUS_OCCUPIED: prefix="Occupied"; break;
		case ID_STATUS_FREECHAT: prefix="FreeChat"; break;
		case ID_STATUS_ONLINE: prefix="On"; break;
		case ID_STATUS_OFFLINE: prefix="Off"; break;
		case ID_STATUS_INVISIBLE: prefix="Inv"; break;
		case ID_STATUS_ONTHEPHONE: prefix="Otp"; break;
		case ID_STATUS_OUTTOLUNCH: prefix="Otl"; break;
		default: return NULL;		
	}
	lstrcpy(str,prefix); lstrcat(str,suffix);
	return(str);
}


void ChangeAllProtoMessages(int statusMode,char *msg)
{
	int protoCount;
	PROTOCOLDESCRIPTOR **proto;

	CallService(MS_PROTO_ENUMPROTOCOLS,(WPARAM)&protoCount,(LPARAM)&proto);
	for(int index=0;index < protoCount;index++)
	{
		CallProtoService(proto[index]->szName,PS_SETAWAYMSG,statusMode,(LPARAM)msg);
	}
}



int Get_Default_Message(char *lpszTitle,char *lpszMsg) // title can be NULL
{
	char szSuffix[DB_SUFFIX_MAX];
	char szSetting[128];

	int iDefMsg = DBGetContactSettingWord(NULL,MOD_NAME,"defmsg",0)+1; // we need +1 so we can use it to get the titles
	sprintf(szSuffix,"%d",iDefMsg);

	strcpy(szSetting,"title");
	strcat(szSetting,szSuffix);

	DBVARIANT dbVar;
	int iRes = DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar);
	
	if ((iRes == 0) && (dbVar.type == DBVT_ASCIIZ))
	{
		if (lpszTitle != NULL)
			strcpy(lpszTitle,dbVar.pszVal);
		DBFreeVariant(&dbVar);

		strcpy(szSetting,"msg");
		strcat(szSetting,szSuffix);
		int iRes = DBGetContactSetting(NULL,MOD_NAME,szSetting,&dbVar);
		
		if ((iRes == 0) && (dbVar.type == DBVT_ASCIIZ))
		{
			strcpy(lpszMsg,dbVar.pszVal);
			DBFreeVariant(&dbVar);
			return(1);
		}
	}

	// get the super-default msg (we don't copy the title too)
	strcpy(lpszMsg,Translate(AWAY_DF_MSG));
	DBFreeVariant(&dbVar);
	return(0);
}


void Interpret_Message_Using_Variables_Plugin(HANDLE hContact, char *lpszMsg)
{
	if (ServiceExists(MS_VARS_FORMATSTRING)) {
	   FORMATINFO fInfo;
	   char *szResult;
	   char errCode[32];
	
	   ZeroMemory(&fInfo, sizeof(fInfo));
	   fInfo.cbSize = sizeof(FORMATINFO);
	   fInfo.szFormat = lpszMsg;
	   szResult = (char *)CallService(MS_VARS_FORMATSTRING, (WPARAM)&fInfo, 0);
	   sprintf(errCode, "errorcode = %d", fInfo.err);
	   if (szResult == NULL) return;
	   //MessageBox(NULL, szResult, errCode,NULL);
	   strcpy(lpszMsg,szResult);
	   CallService(MS_VARS_FREEMEMORY, (WPARAM)szResult, 0);
	}
}

void Interpret_Message(int iStatMode,char *lpszMsg)
{
	// this func does some variable stuff
	for (int index=0, iLen = strlen(lpszMsg);index < iLen;index++)
	{
		if (lpszMsg[index] != '%')
			continue;
		char szReplace[256];
		int iSkip;
		char *szTemp = NULL; // allocated dynamically, seems better
		

		if (!_strnicmp(lpszMsg+index,"%time%",6)) // legacy variables
		{
			GetTimeFormat(LOCALE_USER_DEFAULT,TIME_NOSECONDS,NULL,NULL,szReplace,sizeof(szReplace));
			iSkip = 6;
		}
		else if (!_strnicmp(lpszMsg+index,"%date%",6))
		{
			GetDateFormat(LOCALE_USER_DEFAULT,DATE_SHORTDATE,NULL,NULL,szReplace,sizeof(szReplace));
			iSkip = 6;
		}
		else if(!_strnicmp(lpszMsg+index,"%day%",5))
		{
			GetDateFormat(NULL,0,NULL,"dddd",szReplace,sizeof(szReplace));
			iSkip = 5;
		}
		else if (!_strnicmp(lpszMsg+index,"%statdesc%",10)) // status description
		{
			strcpy(szReplace,(char*)CallService(MS_CLIST_GETSTATUSMODEDESCRIPTION,iStatMode,0));
			iSkip = 10;
		}
		else
			continue;

		// ok, now let's add it in
		int iSecondary = iLen-iSkip-index;
		if (iSecondary > 0)
		{
			szTemp = new char[iSecondary+1];
			strcpy(szTemp,lpszMsg+index+iSkip);
		}
		lpszMsg[index] = '\0'; // take off what's not needed
		strcat(lpszMsg,szReplace);
		if (iSecondary > 0)
		{
			strcat(lpszMsg,szTemp);
			delete [] szTemp;
		}
		index += strlen(szReplace);
		iLen = strlen(lpszMsg); // current length
	}
}

int ICQStatusToGeneralStatus(byte bICQStat)
{
	switch(bICQStat)
	{
		case MTYPE_AUTONA:
		{
			return(ID_STATUS_NA);
		} break;
		case MTYPE_AUTODND:
		{
			return(ID_STATUS_DND);
		} break;
		case MTYPE_AUTOBUSY:
		{
			return(ID_STATUS_OCCUPIED);
		} break;
		case MTYPE_AUTOFFC:
		{
			return(ID_STATUS_FREECHAT);
		} break;
		default:
		{
			return(ID_STATUS_AWAY); // away
		} break;
	}
}


// the allocated data needs to be deallocated when not needed anymore
// this function reads a given line, and allocates room for it and returns the pointer to the
// new space
__forceinline char *Read_Line(FILE *hFile) // get the entire current line and return it
{
	char *lpszLine = NULL;

	long lPos = ftell(hFile);
	char cTemp;

	// simplified
	// could be done nicer, but this works
	while (fread((void*)&cTemp,sizeof(char),1,hFile) == 1)
	{
		if (cTemp == '\n') // find a break line	
		{
			break;
		}
	}

	if ((cTemp != '\n') && (!feof(hFile))) // if this isn't a new line, and this isn't the EOF, it means smt went wrong
	{
		return(NULL);
	}

	// good, allocate it
	long lLen = ftell(hFile) - lPos + 1;

	if ((lLen-1) == 0)
	{
		return(NULL);
	}

	lpszLine = new char[lLen];
	fseek(hFile,lPos,SEEK_SET); // seek back
	fgets(lpszLine,lLen,hFile); // finally

	return(lpszLine);
}


// returns NULL on failure, or a pointer to the filename (needs to be deallocated)
// if everything went right
char *Get_FileLine(const char *lpszFilename,WORD wLine)
{
	// this function was rewrote to address some issues
	// the smaller issue of the absolute/relative paths thingy
	// and the bigger issue, of "unlimited" line length
	char szPath[MAX_PATH];
	char *lpoutData = NULL;

	GetModuleFileName(GetModuleHandle(NULL),szPath,sizeof(szPath));
	char *szEnd = strrchr(szPath,'\\'); // kill the trailing slash, we only need the directoy
	if (szEnd != NULL)
	{
		*(szEnd + 1) = '\0';
	}
	SetCurrentDirectory(szPath); // set it as current direcory

	FILE *fFile = fopen(lpszFilename,"r");

	if (fFile != NULL)
	{
		// ok before we start let's see how many lines we have here
		char cTemp;
		unsigned short sCount = 1; // we always have 1 first
		while (!feof(fFile))
		{
			if ((fread((void*)&cTemp,sizeof(char),1,fFile) == 1) && (cTemp == '\n'))
			{
				sCount++;
			}
		}
		rewind(fFile);

		if ((wLine != 0) && (sCount < wLine)) // hmm doesn't seem right
		{
			fclose(fFile);
		}
		else
		{
			if (wLine == 0) // random
			{
				sCount = (unsigned short)(((float)rand() / (RAND_MAX+1)) * sCount); // random crap
			}
			else
			{
				sCount = wLine - 1;
			}

			unsigned short sCrnt = 0;
			char *lpTempData = NULL;
			while (lpTempData = Read_Line(fFile))
			{
				if (sCrnt++ == sCount)
				{
					// this can't happen in this new version of the function
					// since Read_Line takes care of this
	//				if (lstrlen(lpTempData) > 0) // goodie
	//				{
					char *szLast = lpTempData + lstrlen(lpTempData) - 1;
					if (*szLast == '\n') // line feed
					{
						*szLast = '\0'; // take off the line feed
					}
	//				}

					// might happen after > 0
					if (lstrlen(lpTempData) == 0) // nada
					{
						delete [] lpTempData; // delete it
						lpTempData = NULL; // that way the routine at the end of the function will take care of it
					}

					lpoutData = lpTempData;
					break;
				}
				delete [] lpTempData;
			}

			fclose(fFile);
		}
	}

	// end of function
	if (lpoutData == NULL)
	{
		lpoutData = new char[lstrlen(Translate("Nothing")) + 1];
		lstrcpy(lpoutData,Translate("Nothing"));
	}
	else // process breaklines (\n)
	{
		char *lpBL = NULL;
		while (lpBL = strstr(lpoutData,"\\n"))
		{
			*(lpBL++) = '\r';
			*lpBL = '\n';
		}
	}
	return(lpoutData);
}



// get info. makes post_interpret_message less cluttered
// we always get the name+nick
// time stuff + song name are only received when needed
__forceinline void Get_Current_Info(HANDLE hContact,char *szNick,char *szName,char *szoutSongName,char *szTime,char *szTimePassed/*,char *szRandomLine*/,char *szMsgNum,char *szCNick)
{
	// start with time
	if (szTime != NULL)
	{
		GetTimeFormat(LOCALE_USER_DEFAULT,TIME_NOSECONDS,NULL,NULL,szTime,33);
	}

	if (szTimePassed != NULL)
	{
		DWORD dwDiff = DBGetContactSettingDword(NULL,MOD_NAME,"ctime",0); // temp
		DWORD dwNow = GetTickCount();

		if (dwNow < dwDiff) // funky, but might happen
		{
			dwDiff = 0;
			DBWriteContactSettingDword(NULL,MOD_NAME,"ctime",dwNow);
		}
		dwDiff = dwNow - dwDiff;
		dwDiff /= 1000; // in secs

		if (dwDiff >= 7200) // more than 2 hours
		{
			wsprintf(szTimePassed,Translate("%d hours"),dwDiff / 3600);
		}
		else if (dwDiff >= 120) // more than a 2 minutes
		{
			wsprintf(szTimePassed,Translate("%d minutes"),dwDiff / 60);
		}
		else
		{
			wsprintf(szTimePassed,Translate("%d seconds"),dwDiff);
		}
	}
	
	if (hContact != NULL)
	{
		// find info we need
		DBVARIANT dbVar;
		
		bool anICQContact = false;
		char szcProto[64];
		strcpy(szcProto, (char *) CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM) hContact, 0));
		if(strstr(szcProto,"ICQ"))
		{
			if(DBGetContactSettingDword(hContact,szcProto,"UIN",0) != 0)
				anICQContact = true;
		}
		
		if (anICQContact && DBGetContactSetting(hContact,szcProto,"Nick",&dbVar) == 0)
		{
			if (dbVar.type == DBVT_ASCIIZ)
			{
				// a quick check in the icqOscar protocol (the implementation of which in Miranda, for the matter) turns
				// up that the maximum length for a nickname is like 32 chars... fine ;)
				// who knows, so we're checking anyway

				unsigned int uiLen = (lstrlen(dbVar.pszVal) > 128) ? (128) : (lstrlen(dbVar.pszVal));
				strncpy(szNick,dbVar.pszVal,uiLen);
				szNick[uiLen] = '\0';
			}
			DBFreeVariant(&dbVar);

			if (DBGetContactSetting(hContact,szcProto,"FirstName",&dbVar) == 0)
			{
				if (dbVar.type == DBVT_ASCIIZ)
				{
					// a quick check in the icqOscar protocol (the implementation of which in Miranda, for the matter) turns
					// up that the maximum length for a nickname is like 32 chars... fine ;)
					// who knows, so we're checking anyway

					unsigned int uiLen = (lstrlen(dbVar.pszVal) > 128) ? (128) : (lstrlen(dbVar.pszVal));
					strncpy(szName,dbVar.pszVal,uiLen);
					szName[uiLen] = '\0';
				}
				else
				{
					strcpy(szName,szNick);
				}
				DBFreeVariant(&dbVar);
			}
			else
			{
				strcpy(szName,szNick);
			}
		}
		else// if (DBGetContactSetting(hContact,"MSN","Nick",&dbVar) == 0) // try MSN stuff
		{
			// to make this work with both MSN and Yahoo, let's do it this way
			strcpy(szNick,(char*)CallService(MS_CLIST_GETCONTACTDISPLAYNAME,(WPARAM)hContact,GCDNF_NOMYHANDLE));
			strcpy(szName,szNick);
		}
	}


	if (szoutSongName != NULL)
	{
		HWND hwndWinamp = FindWindow("Winamp v1.x",NULL);

		// this is a modified version of a code segment from nullsoft's nsdn
		// guessing the maximum size of a song name in winamp is 2048 chars
		// good enough for me ;)
		if (hwndWinamp != NULL) // got it (Winamp 2.x)
		{
			if (SendMessage(hwndWinamp,WM_USER,0,104) == 1) // a song is playing
			{
				char szSongName[2048];
				GetWindowText(hwndWinamp,szSongName,sizeof(szSongName));

				int iLen = lstrlen(szSongName);

				if (iLen > 9) // at least 9-10
				{
					for (int index = iLen-1;index >= 0;index--)
					{
						if (_strnicmp(szSongName+index," - Winamp",9) == 0) // got it
						{
							szSongName[index] = '\0';
							break;
						}
					}

					if (index != -1) // i.e. we kicked off middle way
					{
						// we're gonna cut off the '(song_num). ' before the song name - it's so annoying
						char *lpCopyFrom = strchr(szSongName,' ');

						if (lpCopyFrom)
						{
							iLen = lstrlen(szSongName) - (++lpCopyFrom - szSongName); // recalc
							if (iLen > 0)
							{
								if (iLen > 512) // we can't afford to copy anymore
								{
									iLen = 512;
								}
								strncpy(szoutSongName,lpCopyFrom,iLen);
								szoutSongName[iLen] = '\0';
							}
						}
					}
				}
			}
		}
		else // NULL, let's try Winamp3
		{
			hwndWinamp = FindWindow("STUDIO",NULL);

			if (hwndWinamp != NULL)
			{
				char szSongName[2048];
				GetWindowText(hwndWinamp,szSongName,sizeof(szSongName));

				int iLen = lstrlen(szSongName);

				if (iLen > 10) // at least 10-11
				{
					for (int index = iLen-1;index >= 0;index--)
					{
						if (_strnicmp(szSongName+index," (playing)",10) == 0) // got it
						{
							szSongName[index] = '\0';
							break;
						}
					}

					if (index != -1) // i.e. we kicked off middle way
					{
						// we're gonna cut off the '(song_num). ' before the song name - it's so annoying
						char *lpCopyFrom = strchr(szSongName,' ') + 1;

						iLen = lstrlen(szSongName) - (lpCopyFrom - szSongName); // recalc
						if (iLen > 0)
						{
							if (iLen > 512) // we can't afford to copy anymore
							{
								iLen = 512;
							}
							strncpy(szoutSongName,lpCopyFrom,iLen);
							szoutSongName[iLen] = '\0';
						}
					}
				}
			}
		}
	}

	if (hContact != NULL)
	{
		if (szMsgNum != NULL)
		{
			sprintf(szMsgNum,"%d",DBGetContactSettingWord(hContact,MOD_NAME,"msgnum",0));
		}

		if (szCNick != NULL)
		{
			DBVARIANT dbVar;
			if (DBGetContactSetting(hContact,"CList","MyHandle",&dbVar) == 0)
			{
				if (dbVar.type == DBVT_ASCIIZ)
				{
					strcpy(szCNick,dbVar.pszVal);
				}
				else
				{
					strcpy(szCNick,szNick);
				}
				DBFreeVariant(&dbVar);
			}
			else
			{
				strcpy(szCNick,szNick);
			}
		}
	}
}


void Post_Interpret_Message(char *lpszMsg,HANDLE hContact)
{
	// pre-processing to get winamp's song name + first name of contact + nickname of contact
	char szNick[129], szName[129], szSongName[513], szTime[33], szTimePassed[33];//, szRandomLine[513];
	char szReplace[513];
	char szMsgNum[6];
	char szCNick[129];
	char szmyNick[129];
	DBVARIANT dbvp,dbvo;

	if (DBGetContactSetting(NULL,MOD_NAME,"prefix",&dbvp) == 0 && dbvp.type==DBVT_ASCIIZ) {
				char szTmp[AWAY_MSGDATA_MAX+1];
				strcpy(szTmp, dbvp.pszVal);
				strcat(szTmp, lpszMsg);
				strcpy(lpszMsg, szTmp);
				DBFreeVariant(&dbvp);
				//delete [] szTmp;
	}
	
	//Get my nick on this protocol
	// default : Stranger
	strcpy(szmyNick,Translate("Stranger"));
	if(DBGetContactSettingByte(NULL,MOD_NAME,"defnick",1))
	{
		strcpy(szmyNick,(char*)CallService(MS_CLIST_GETCONTACTDISPLAYNAME,NULL,0));
	}
	else if(hContact != NULL)
	{
		char szcProto[64];
  		strcpy(szcProto, (char *) CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM) hContact, 0));
		if (DBGetContactSetting(NULL, szcProto,"Nick",&dbvo) == 0 && dbvo.type==DBVT_ASCIIZ && szcProto != NULL) {
			strcpy(szmyNick, dbvo.pszVal );
			DBFreeVariant(&dbvo); // important, free the memory
		}
		//delete [] szcProto;
	}
	else
	{
		strcpy(szmyNick,(char*)CallService(MS_CLIST_GETCONTACTDISPLAYNAME,NULL,0));
	}
	
	strcpy(szNick,Translate("Stranger"));
	strcpy(szName,szNick);
	strcpy(szCNick,szNick);
	strcpy(szSongName,Translate("Nothing"));
	szTime[0] = '\0';
	szTimePassed[0] = '\0';
	strcpy(szMsgNum,"0");
	byte bSkip;

	// let's get all info.
	// there's no point in all of these tests, we might as well get everything
	// only %randomline% is tested
	Get_Current_Info(hContact,szNick,szName,szSongName,szTime,szTimePassed,/*(strstr(lpszMsg,"%randomline%") != NULL) ? (szRandomLine) : (NULL),*/szMsgNum,szCNick);

	char *szPos = NULL;
	do
	{
		
		if (szPos = strstr(lpszMsg,"%yournick%"), szPos != NULL)
		{
			strcpy(szReplace,szNick);
			bSkip = 10;
		}
		else if (szPos = strstr(lpszMsg,"%mynick%"), szPos != NULL) // nickname
		{
			strcpy(szReplace,szmyNick);
			bSkip = 8;
		}
		else if (szPos = strstr(lpszMsg,"%yourname%"), szPos != NULL)
		{
			strcpy(szReplace,szName);
			bSkip = 10;
		}
		else if (szPos = strstr(lpszMsg,"%currenttime%"), szPos != NULL)
		{
			strcpy(szReplace,szTime);
			bSkip = 13;
		}
		else if (szPos = strstr(lpszMsg,"%timepassed%"), szPos != NULL)
		{
			strcpy(szReplace,szTimePassed);
			bSkip = 12;
		}
		else if (szPos = strstr(lpszMsg,"%randomline%"), szPos != NULL) // this is still here for compatibility reasons, nothing else
		{
			char *szTemp = Get_FileLine("as_random.txt",0);
			strcpy(szReplace,szTemp);
			delete [] szTemp;
			bSkip = 12;
		}
		else if (szPos = strstr(lpszMsg,"%randomline("), szPos != NULL) // new format - %randomline(filename)
		{
			char szFilename[MAX_PATH + 1];
			char *lpEnd = strchr(szPos+12,')');
			if ((lpEnd == NULL) || (lpEnd == szPos+12)) // doesn't make any sense (not found, or very close to first one)
			{
				strcpy(szFilename,"as_random.txt"); // assume the user is an idiot who can't follow a simple syntax... :)
			}
			else
			{
				strncpy(szFilename,szPos+12,lpEnd - (szPos+12));
				szFilename[lpEnd - (szPos+12)] = '\0';
				/*
				// find slashes
				// hope this works
				char *lpSlash = strstr(szFilename,"..\\");
				if (lpSlash == NULL) // try backslash
				{
					lpSlash = strstr(szFilename,"../");
				}
				// we only disable outwards traversing (I think I just made that up)
				// inwards is fine
				if (lpSlash != NULL)
				{
					strcpy(szFilename,"as_random.txt"); // the all-mighty
				}*/
				// revenge of the slashes have stopped
				// a user can use any slashes he wants
				// who cares
				// absolute paths now also work, due to the new Get_FileLine function
			}

			bSkip = 14 + lstrlen(szFilename); // "%randomline(" + the file len + ")%" (we get it now coz szTemp will change later)
			char *szTemp = Get_FileLine(szFilename,0);
			strcpy(szReplace,szTemp);
			delete [] szTemp;
		}
		else if (szPos = strstr(lpszMsg,"%singleline("), szPos != NULL) // variation of %randomline%
		{
			char szFilename[MAX_PATH + 1]; // will hold the filename for us, plus it will also hold the line when we're done (a byte saved is a byte for... Windows to waste, or smt like that)
			char szTLine[10];
			char *lpComma = strchr(szPos+12,',');

			if ((lpComma == NULL) || (lpComma == szPos+12))
			{
				strcpy(szFilename,"as_random.txt");
				strcpy(szTLine,"1");
			}
			else
			{
				// get file
				strncpy(szFilename,szPos+12,lpComma - (szPos+12));
				szFilename[lpComma - (szPos+12)] = '\0';

				char *lpEnd = strchr(lpComma+1,')');

				if ((lpEnd == NULL) || (lpEnd == lpComma+1))
				{
					strcpy(szTLine,"1");
				}
				else
				{
					// get line
					strncpy(szTLine,lpComma+1,lpEnd - (lpComma+1));
					szTLine[lpEnd - (lpComma+1)] = '\0';
				}
			}

			bSkip = 15 + lstrlen(szFilename) + lstrlen(szTLine); // "%singleline(" + the file len + "," + line num + ")%" (we get it now coz szTemp will change later)
			WORD wLine = atoi(szTLine);
			if (wLine == 0) wLine = 1; // yup
			char *szTemp = Get_FileLine(szFilename,wLine);
			strcpy(szReplace,szTemp);
			delete [] szTemp;
		}
		else if (szPos = strstr(lpszMsg,"%winampsong%"), szPos != NULL)
		{
			strcpy(szReplace,szSongName);
			bSkip = 12;
		}
		else if (szPos = strstr(lpszMsg,"%messagenum%"), szPos != NULL)
		{
			strcpy(szReplace,szMsgNum);
			bSkip = 12;
		}
		else if (szPos = strstr(lpszMsg,"%yourcnick%"), szPos != NULL)
		{
			strcpy(szReplace,szCNick);
			bSkip = 11;
		}

		if (szPos != NULL)
		{
			// ok, now let's add it in
			char *szTemp = NULL;
			int iSecondary = lstrlen(lpszMsg)-bSkip-(szPos - lpszMsg);
			if (iSecondary > 0)
			{
				szTemp = new char[iSecondary+1];
				if (szTemp == NULL)
				{
					break;
				}
				strcpy(szTemp,szPos + bSkip);
			}
			*szPos = '\0'; // take off what's not needed
			strcat(lpszMsg,szReplace);
			if (iSecondary > 0)
			{
				strcat(lpszMsg,szTemp);
				delete [] szTemp;
			}
		}
	} while (szPos != NULL);
}